import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;

/**
 * A single-purpose command line tool for splitting a long panorama picture
 * into horizontal tiles.
 *
 */
public final class ImageSprite {

    /** Width/height-ratio of the generated tiles. */
    private static final double TILE_RATIO = 8.0 / 3.0;

    /** Number of tiles to generate. */
    private final int nTiles;

    /** Panorama picture to be tiled. */
    private final File panoramaFile;

    /** Format string for the output files (contains exactly one {@code %d}
     * specifier). */
    private final String fileNameScheme;

    /** Image format for the output images. */
    private final String format;

    /**
     * Create a new instance.
     *
     * @param src
     *         file name of the panorama picture to read in
     * @param dst
     *         {@code printf} format string for the output file names
     * @param n
     *         number of tiles to generate
     *
     */
    private ImageSprite(final String src,
                        final String dst,
                        final int n) {
        this.panoramaFile = new File(src);
        this.fileNameScheme = dst;
        this.nTiles = n;
        {
            final String[] words = dst.split("[.]");
            this.format = words[words.length - 1].toUpperCase();
        }
    }

    /**
     * Process the input and write the output.
     *
     * @throws IOException
     *         if an I/O error occurs
     *
     */
    private void process() throws IOException {
        final BufferedImage panorama = ImageIO.read(this.panoramaFile);
        final int height = panorama.getHeight();
        final int width = (int) (ImageSprite.TILE_RATIO * height);
        final int shift = (panorama.getWidth() - width) / (this.nTiles - 1);
        for (int i = 0; i < this.nTiles; i++) {
            final int x = i * shift;
            final int y = 0;
            final String filename = String.format(this.fileNameScheme, i);
            final BufferedImage subpic = panorama.getSubimage(
                    x, y, width, height);
            if (!ImageIO.write(subpic, this.format, new File(filename))) {
                final String msg = String.format(
                        "cannot encode format '%s'", this.format);
                throw new IOException(msg);
            }
        }
    }

    /**
     * Run the program.
     *
     * @param args
     *         command line arguments
     * @throws IOException
     *         if an I/O error occurs
     *
     */
    public static void main(final String[] args) throws IOException {
        for (final String arg : args) {
            if ("--help".equals(arg)) {
                System.out.printf("usage: java %s SOURCE SCHEME N%n",
                                  ImageSprite.class.getCanonicalName());
                System.out.println("");
                System.out.println(
                        "Horizontally split source image SOURCE into N images and save them as\n"
                        + "files as specified in SCHEME which should be a prinf-style format\n"
                        + "string with exactly one '%d' specifier.\n");
                System.exit(0);
            }
        }
        if (args.length < 3) {
            System.err.println("error: too few arguments");
            System.exit(2);
        } else if (args.length > 3) {
            System.err.println("error: too few arguments");
            System.exit(2);
        } else {
            final int n = Integer.parseInt(args[2]);
            final ImageSprite cs = new ImageSprite(args[0], args[1], n);
            cs.process();
        }
    }
}
